function ghandle = affiche(s,ghandle)

% display results of measure matching algorithm
% usage: 
%   affiche(s);            s is the output structure of a call to measmatch
%   ghandle = affiche(s);  to store the graphical handles
%   affiche(s,ghandle);    to update the current figure
%
% author : Joan Glauns - joan.glaunes@free.fr

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%% display parameters %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

showminim = 1;                  % if 1 plot functional vs iterations 
showtitle = 1;                  % if 1 write parameters on figure
showx = 1;                      % if 1 plot template data
xcolor = [0;0;1];               % color for template data
showxrig = 0;                   % if 1 plot prior rigid transforamtion of template
xrigcolor = [1;1;1];            % color for xrig
showy = 1;                      % if 1 plot target data
ycolor = [.1;.5;0];            % color for target data
showphi = 1;                    % if 1 plot deformation of template
phicolor = [1;0;0];           % color for deformation of template
showtraj = 0;                   % plot trajectories
showbyuS = 0;                   % display template byu surface
byuScolor = [0;1;0];            % template byu color
byuS = 0;                       % template byu file name
showbyuD = 0;                   % display deformed byu surface
byuDcolor = [92;33;165]/256;    % deformed byu color
byuD = 0;                       % file name of deformed byu surface
vbyuD = [0,0,0];                % displacement vector for byuD
showbyuR = 0;                   % plot template byu surface with rigid registration
byuRcolor = [92;33;165]/256;    % color of byuR
vbyuR = 2*vbyuD;                % displacement vector for byuR
showbyuT = 0;                   % show target byu surface
byuTcolor = [168;30;30]/256;    % color of byuT 
byuT = 0;                       % target byu surface name
showcurv = 'nocurv';            % show curvature map for byu surfaces ('curv' or 'nocurv')
transparent = 0;                % if 1 add transparency on byu surfaces
showinitmom = 0;                % show initial moment on byuR
showgrid = 0;                   % display grid
gridsize = 30;                  % size of grid

showweights = 0;                % display weight of each landmark
showgroups = 0;                 % display group of each landmark
viewvector = 0;                 % 3D display view vector
axisvector = 0;                 % 3D display axis vector
updategraph = 0;                % only update deformed data
printag = 0;                    % if 1 save figure
printname = 's';                % file name to save figure
printmode = '-depsc';           % file type
printext = '.eps';              % file extension
printopt = '';                  % additional printing options
showlegend = 1;                 % if 1 print legend

% load variables of structure s
loadstruct('s')

% save all variables in structure s
savestruct('s',who);

% groups symbols
grpsgn = {'o','d','x','+','*'};
if ng > 5
    warning('add more group signs');
end

if nargin==1
    ghandle = 0;
end

%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% update mode %%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%

if isstruct(ghandle)
        
    if showminim
        subplot(6,3,17)
        hold on
        if length(J) > 1
            plot(J,'k')
        end
        subplot(1.2,1,1)
    end
    
    if showtraj
        delete(ghandle.traj)
        ghandle.traj = plot3(X1',X2',X3','b');       % plot trajectories in blue
    end
    
    if showphi
        for k = 1:ng
            ind = gtabx(k):gtabx(k+1)-1;
            set(ghandle.phi(k),'XData',X1(ind,T)');
            set(ghandle.phi(k),'YData',X2(ind,T)');
            set(ghandle.phi(k),'ZData',X3(ind,T)');
        end
    end
    
    if showbyuD
        [ptset,ptface] = readbyu(byuS);
        ptset = flow3fast(s,ptset);
        ptsetf = ptset(:,ptface);
        ptlg = size(ptsetf,2)/3;
        set(ghandle.byuD,'XData',reshape(ptsetf(1,:)+vbyuD(1),3,ptlg));
        set(ghandle.byuD,'YData',reshape(ptsetf(2,:)+vbyuD(2),3,ptlg));
        set(ghandle.byuD,'ZData',reshape(ptsetf(3,:)+vbyuD(3),3,ptlg));
        if strcmp(showcurv,'curv')
            C = meancurv(ptset,ptface);
            stdC = std(C);
            mnC = mean(C);
            lim = 1;
            minlim = mean(C)-std(C)*lim;
            maxlim = mean(C)+std(C)*lim;
            C(find(C<minlim)) = minlim;
            C(find(C>maxlim)) = maxlim;
            C = C - min(C);
            C = C / max(C);            
            C = shiftdim(reshape(C(ptface),3,ptlg),-1);
            C = [byuDcolor(1)*C;byuDcolor(2)*C;byuDcolor(3)*C];
            C = shiftdim(C,1);
            set(ghandle.byuD,'CData',C);
        end
    end  
    
    if showgrid
        delete(ghandle.grid)
        ghandle.grid = grid3(s);
    end

    
else    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%
    %%%% normal mode  %%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%
    
    clf
    
    if showminim
        subplot(6,3,17)
        hold on
        if length(J) > 1
            plot(J,'k')
        end
        subplot(1.2,1,1)
    end
    
    hold on
    
    if showtraj
        ghandle.traj = plot3(X1',X2',X3','b');       % plot trajectories in blue
    end
    
    legh = [];

    if showx
        for k = 1:ng
            % plot initial data
            ind = gtabx(k):gtabx(k+1)-1;
            h = plot3(x(1,ind),x(2,ind),x(3,ind),grpsgn{k});
            set(h,'MarkerSize',5,'Color',xcolor,'MarkerFaceColor',xcolor);
            set(h,'Tag','initial data');
            legh = [legh,h];
        end
    end
    
    xrig = transmatrix * x + repmat(transvector,1,nx);
    
    if showxrig
        for k = 1:ng
            % plot rigid transformation
            ind = gtabx(k):gtabx(k+1)-1;
            h = plot3(xrig(1,ind),xrig(2,ind),xrig(3,ind),grpsgn{k});
            set(h,'MarkerSize',5,'Color',xrigcolor,'MarkerFaceColor',xrigcolor);
            set(h,'Tag','rigid deformation');
            legh = [legh,h];
        end
    end

    if showy
        for k = 1:ng
            % plot final data
            ind = gtaby(k):gtaby(k+1)-1;
            h = plot3(y(1,ind),y(2,ind),y(3,ind),grpsgn{k});
            set(h,'MarkerSize',5,'Color',ycolor,'MarkerFaceColor',ycolor);
            set(h,'Tag','target');
            legh = [legh,h];
        end
        if showweights
            for n=1:ny
                text(y(1,n),y(2,n),y(3,n),[' ',num2str(wy(n))])
            end
        end
        if showgroups
            for n=1:ny
                text(y(1,n),y(2,n),y(3,n),[' ',num2str(grpy(n))])
            end
        end
    end

    if showphi
        ghandle.phi = zeros(1,ng);
        for k = 1:ng
            % plot deformed data
            ind = gtabx(k):gtabx(k+1)-1;
            ghandle.phi(k) = plot3(X1(ind,T),X2(ind,T),X3(ind,T),grpsgn{k});
            set(ghandle.phi(k),'MarkerSize',5,'Color',phicolor,'MarkerFaceColor',phicolor);
            set(ghandle.phi(k),'Tag','elastic registration');
            legh = [legh,ghandle.phi(k)];
        end
        if showweights
            for n=1:nx
                text(X1(n,T),X2(n,T),X3(n,T),[' ',num2str(wx(n))])
            end
        end
        if showgroups
            for n=1:nx
                text(X1(n,T),X2(n,T),X3(n,T),[' ',num2str(grpx(n))])
            end
        end
    end
        
    if showgrid
        ghandle.grid = grid3(s);
    end
    
    if showbyuT
        h = plotbyu(byuT,showcurv,byuTcolor);
        set(h,'Tag','target surface');
        legh = [legh,h];
        ghandle.byuT = h;
    end
    
    if showbyuS
        h = plotbyu(byuS,showcurv,byuScolor);
        legh = [legh,h];
        set(h,'Tag','source surface');
    end
    
    if showbyuR
        [ptset,ptface] = readbyu(byuS);
        % additional affine transformation
        ptset = transmatrix * ptset + repmat(transvector,1,size(ptset,2));
        ptset = ptset(:,ptface);
        ptlg = size(ptset,2)/3;
        if showinitmom
            byuRcolor = normalmom(s,byuS);
        end
        h = plotbyu(byuS,showcurv,byuRcolor);
        set(h,'XData',reshape(ptset(1,:),3,ptlg));
        set(h,'YData',reshape(ptset(2,:),3,ptlg));
        set(h,'ZData',reshape(ptset(3,:),3,ptlg));
        legh = [legh,h];
        set(h,'Tag','template surface');
        ghandle.byuR = h;
    end        
    
    if showbyuD
        [V,F] = readbyu(byuS);
        ptset = flow3(s,V);
        ptsetf = ptset(:,F);
        ptlg = size(ptsetf,2)/3;
        h = plotbyu(byuS,showcurv,byuDcolor);
        set(h,'XData',reshape(ptsetf(1,:)+vbyuD(1),3,ptlg));
        set(h,'YData',reshape(ptsetf(2,:)+vbyuD(2),3,ptlg));
        set(h,'ZData',reshape(ptsetf(3,:)+vbyuD(3),3,ptlg));
        if strcmp(showcurv,'curv')
            C = meancurv(ptset,F);
            stdC = std(C);
            mnC = mean(C);
            lim = 1;
            minlim = mean(C)-std(C)*lim;
            maxlim = mean(C)+std(C)*lim;
            C(find(C<minlim)) = minlim;
            C(find(C>maxlim)) = maxlim;
            C = C - min(C);
            C = C / max(C);            
            C = shiftdim(reshape(C(abs(F)),3,ptlg),-1);
            C = [byuDcolor(1)*C;byuDcolor(2)*C;byuDcolor(3)*C];
            C = shiftdim(C,1);
            set(h,'CData',C);
        end
        legh = [legh,h];
        set(h,'Tag','elastic registration');
        ghandle.byuD = h;
    end   
    
    if showlegend & legh
        legend(legh,get(legh,'Tag'))
    end
            
    if size(viewvector) ~= [1,1]
        view(viewvector);
    else
        viewvector = view;
    end
    
    axisvector = 0;
    if size(axisvector) ~= [1,1]
        axis(axisvector);
    else
        axisvector = axis;
    end
    
    axis equal
    %axis tight
    
end

if showtitle
    if min(sigmaV) < max(sigmaV)
        sigmaVstr = [num2str(max(sigmaV)),' -> ',num2str(min(sigmaV))];
    else
        sigmaVstr = num2str(sigmaV(1));
    end
    title(['sigmaV=',sigmaVstr,', ','sigmaI=',num2str(sigmaI),', ',...
            'gammaR=',num2str(gammaR),', ',num2str(length(J)),' itrations']);
end

if printag
    print(printopt,printmode,[printname,'.it',num2str(1000+length(J)),printext]);
end

if transparent
    % set transparency rendering
    set(gcf,'Renderer','zbuffer');
    alpha(ghandle.byuT,.2);
    alpha(ghandle.byuD,.4);
    alpha(ghandle.byuR,.2);
    shading interp
else
    camlight
    shading interp
end

drawnow
shg

function h = grid3(s)
% construction et affichage de la grille 3D
loadstruct('s')
gridstep = max(max([x,y]) - min([x,y]))/gridsize;
g1 = min([x(1,:),y(1,:)]):gridstep:max([x(1,:),y(1,:)]);
s1 = length(g1);
g2 = min([x(2,:),y(2,:)]):gridstep:max([x(2,:),y(2,:)]);
s2 = length(g2);
g3 = min([x(3,:),y(3,:)]):gridstep:max([x(3,:),y(3,:)]);
s3 = length(g3);
g1 = repmat(g1',1,s2*s3);
g2 = kron(g2,ones(s3,s1));
g3 = kron(ones(s1,s2),g3);
g1 = g1(:)';
g2 = g2(:)';
g3 = g3(:)';
g = flow(s,[g1;g2;g3]);
g1 = reshape(g(1,:),s1,s2*s3);
g2 = reshape(g(2,:),s1,s2*s3);
g3 = reshape(g(3,:),s1,s2*s3);
h = [];
if s1 > 1
    h = [h;plot3(g1,g2,g3,'k')];
end
g1 = g1';
g1 = reshape(g1,s3,s1*s2);
g2 = g2';
g2 = reshape(g2,s3,s1*s2);
g3 = g3';
g3 = reshape(g3,s3,s1*s2);
if s3 > 1
    h = [h;plot3(g1,g2,g3,'k')];
end
g1 = g1';
g1 = reshape(g1,s2,s1*s3);
g2 = g2';
g2 = reshape(g2,s2,s1*s3);
g3 = g3';
g3 = reshape(g3,s2,s1*s3);
if s2 > 1
    h = [h;plot3(g1,g2,g3,'k')];        
end
set(h,'Color',.6*[1,1,1]);

